using System;
using System.IO;

namespace Classless.MXO.Blipper {
	/// <summary>A class representing a single blip in the file.</summary>
	public class Blip {
		private string _name;
		private double _x;
		private double _y;
		private double _z;
		private DistrictType _district;


		/// <summary>Gets or sets the name used to identify the blip.</summary>
		public string Name {
			get { return _name; }
			set {
				if (value.Length > ushort.MaxValue) {
					_name = value.Substring(0, ushort.MaxValue);
				} else {
					_name = value;
				}
			}
		}

		/// <summary>Gets or sets the X coordinate of the blip.</summary>
		public double X {
			get { return _x; }
			set { _x = value; }
		}

		/// <summary>Gets or sets the Y coordinate of the blip.</summary>
		public double Y {
			get { return _y; }
			set { _y = value; }
		}

		/// <summary>Gets or sets the Z coordinate of the blip.</summary>
		public double Z {
			get { return _z; }
			set { _z = value; }

		}

		/// <summary>Gets or sets the District the coordinates of the blip are in.</summary>
		public DistrictType District {
			get { return _district; }
			set { _district = value; }
		}


		/// <summary>Initializes a new instance of Blip.</summary>
		public Blip() {
			_name = string.Empty;
			_district = DistrictType.RichlandWestview;
		}

		/// <summary>Initializes a new instance of Blip.</summary>
		/// <param name="s">The stream to read the blip from.</param>
		public Blip(Stream s) {
			// Find out how long the name is.
			byte[] buffer = new byte[2];
			s.Read(buffer, 0, 2);
			int nameLength = (int)(BitConverter.ToInt16(buffer, 0)) * 2;

			// Get the name.
			byte[] nameBuffer = new byte[nameLength];
			s.Read(nameBuffer, 0, nameLength);
			Name = System.Text.Encoding.Unicode.GetString(nameBuffer, 0, nameLength);

			// Chew off some bytes that appear to be some type of padding.
			s.Read(buffer, 0, 1);
			s.Seek(((buffer[0] * 2) + 1), SeekOrigin.Current);

			// Get the coordinates.
			byte[] coordBuffer = new byte[8];
			s.Read(coordBuffer, 0, 8);
			X = BitConverter.ToDouble(coordBuffer, 0);
			s.Read(coordBuffer, 0, 8);
			Y = BitConverter.ToDouble(coordBuffer, 0);
			s.Read(coordBuffer, 0, 8);
			Z = BitConverter.ToDouble(coordBuffer, 0);

			// Get the district.
			byte[] districtBuffer = new byte[4];
			s.Read(districtBuffer, 0, 4);
			try { District = (DistrictType)(BitConverter.ToUInt32(districtBuffer, 0)); } catch (InvalidCastException) { /* Just in case. */ }
		}


		/// <summary>Writes the series of bytes that represent the blip.</summary>
		/// <param name="s">The stream to write the bytes to.</param>
		public void Write(Stream s) {
			byte[] paddingBytes = new byte[4] { 0x01, 0x00, 0x20, 0x00 };

			// Write the name length.
			s.Write(BitConverter.GetBytes((ushort)(Name.Length)), 0, 2);

			// Write the name.
			s.Write(System.Text.Encoding.Unicode.GetBytes(Name), 0, (Name.Length * 2));

			// Write the bytes we haven't figured out yet.
			s.Write(paddingBytes, 0, paddingBytes.Length);

			// Write the coordinates.
			s.Write(BitConverter.GetBytes(X), 0, 8);
			s.Write(BitConverter.GetBytes(Y), 0, 8);
			s.Write(BitConverter.GetBytes(Z), 0, 8);

			// Write the district.
			s.Write(BitConverter.GetBytes((uint)District), 0, 4);
		}
	}
}
