using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.IO;

namespace Classless.MXO.Blipper {
	/// <summary>The main window of the application.</summary>
	public class FrmMain : Form {
		#region Control Declarations
		private System.Windows.Forms.MainMenu mnuMain;
		private System.Windows.Forms.MenuItem mnuFileOpen;
		private System.Windows.Forms.MenuItem mnuFileSave;
		private System.Windows.Forms.MenuItem mnuFileNew;
		private System.Windows.Forms.MenuItem mnuFileSaveAs;
		private System.Windows.Forms.MenuItem menuItem6;
		private System.Windows.Forms.MenuItem mnuFileExit;
		private System.Windows.Forms.MenuItem mnuHelp;
		private System.Windows.Forms.MenuItem mnuHelpAbout;
		private System.Windows.Forms.MenuItem mnuBlipsMainImport;
		private System.Windows.Forms.MenuItem menuItem2;
		private System.Windows.Forms.MenuItem mnuBlipsMainAdd;
		private System.Windows.Forms.MenuItem mnuBlipsMainEdit;
		private System.Windows.Forms.MenuItem mnuBlipsMainRemove;
		private System.Windows.Forms.MenuItem menuItem3;
		private System.Windows.Forms.MenuItem menuItem1;
		private System.Windows.Forms.ContextMenu mnuBlips;
		private System.Windows.Forms.MenuItem mnuBlipsEdit;
		private System.Windows.Forms.MenuItem mnuBlipsRemove;
		private System.Windows.Forms.ListView listBlips;
		private System.Windows.Forms.ColumnHeader colBlipsName;
		private System.Windows.Forms.ColumnHeader colBlipsDistrict;
		private System.Windows.Forms.ColumnHeader colBlipsX;
		private System.Windows.Forms.ColumnHeader colBlipsY;
		private System.Windows.Forms.ColumnHeader colBlipsZ;
		private System.Windows.Forms.OpenFileDialog dialogOpen;
		private System.Windows.Forms.SaveFileDialog dialogSave;
		private System.ComponentModel.Container components = null;
		#endregion


		private BlipCollection _blipCollection;
		private string _blipFile;
		private bool _modified;
		private ListViewColumnSorter lvColumnSorter;


		/// <summary>Gets or sets the blip filename.</summary>
		private string BlipFile {
			get { return _blipFile; }
			set {
				_blipFile = value;
				if (_blipFile.Length == 0) {
					this.Text = "Blipper";
				} else {
					this.Text = "Blipper - " + Path.GetFileName(_blipFile);
				}
			}
		}


		/// <summary>Initializes a new instance of FrmMain.</summary>
		public FrmMain() {
			InitializeComponent();
			StartEventProcessing();
			NewFile();

			lvColumnSorter = new ListViewColumnSorter();
			listBlips.ListViewItemSorter = lvColumnSorter;
		}


		/// <summary>Clean up any resources being used.</summary>
		/// <param name="disposing"></param>
		protected override void Dispose(bool disposing) {
			if ((disposing) & (components != null)) {
				components.Dispose();
			}
			base.Dispose(disposing);
		}


		/// <summary>Starts a new blip file.</summary>
 		public void NewFile() {
			_blipCollection = new BlipCollection();
			BlipFile = string.Empty;
			_modified = false;
			refreshList();
		}


		/// <summary>Opens a blip file.</summary>
		/// <param name="filename">The file to open.</param>
		public void OpenFile(string filename) {
			try {
				// Read in the file.
				FileStream fs = null;
				try {
					fs = new FileStream(filename, FileMode.Open, FileAccess.Read, FileShare.Read);
					_blipCollection = new BlipCollection(fs);
				} catch {
					throw;
				} finally {
					if (fs != null) { fs.Close(); }
				}

				BlipFile = filename;
				_modified = false;

				// Update the list.
				refreshList();
			} catch (Exception ex) {
				MessageBox.Show("There was an error when opening the file." + Environment.NewLine + Environment.NewLine + ex.Message, "File Error");
			}
		}


		/// <summary>Saves the blip file.</summary>
		/// <returns>true if the save was successful; false if it wasn't.</returns>
		public bool SaveFile() {
			if (BlipFile.Length == 0) {
				return SaveFileAs();
			}

			try {
				saveToFile(BlipFile);
				return true;
			} catch {
				return SaveFileAs();
			}
		}


		/// <summary>Saves the blip file with a name of the user's chosing.</summary>
		/// <returns>true if the save was successful; false if it wasn't.</returns>
		public bool SaveFileAs() {
			if (BlipFile.Length == 0) {
				dialogSave.FileName = "pblip.cfg";
			} else {
				dialogSave.FileName = BlipFile;
			}

			if (dialogSave.ShowDialog(this) != DialogResult.Cancel) {
				try {
					saveToFile(dialogSave.FileName);
					BlipFile = dialogSave.FileName;
					return true;
				} catch (Exception ex) {
					MessageBox.Show("There was an error when saving the file." + Environment.NewLine + Environment.NewLine + ex.Message, "File Error");
					return false;
				}
			} else {
				return false;
			}
		}


		/// <summary>Updates the listbox with the contents of the blip collection.</summary>
 		private void refreshList() {
			listBlips.Items.Clear();
			foreach (Blip b in _blipCollection.Blips) {
				addBlipToList(b);
			}
		}


		/// <summary>Adds a blip to the listbox.</summary>
		/// <param name="b">The blip to add.</param>
		private void addBlipToList(Blip b) {
			ListViewItem lvi = new ListViewItem(b.Name);
			lvi.SubItems.Add(b.District.ToString());
			lvi.SubItems.Add(b.X.ToString());
			lvi.SubItems.Add(b.Y.ToString());
			lvi.SubItems.Add(b.Z.ToString());
			lvi.Tag = b;
			listBlips.Items.Add(lvi);
		}


		/// <summary>Checks to see if the file needs to be saved.</summary>
		/// <returns>true if the action can continue; false if it shouldn't.</returns>
		private bool ensureSave() {
			if (_modified) {
				DialogResult dr = MessageBox.Show("You have modified the current file, would you like to save it first?", "File Modified", MessageBoxButtons.YesNoCancel);

				if (dr == DialogResult.Cancel) {
					return false;
				} else if (dr == DialogResult.Yes) {
					return SaveFile();
				}
			}

			return true;
		}


		/// <summary>Writes the blip file to disk.</summary>
		/// <param name="filename">The filename to use.</param>
		private void saveToFile(string filename) {
			// Make a backup.
			if (File.Exists(filename)) {
				File.Copy(filename, filename + ".old", true);
			}

			// Write to the disk.
			FileStream fs = null;
			try {
				fs = new FileStream(filename, FileMode.Create, FileAccess.Write, FileShare.Write);
				_blipCollection.Write(fs);
			} catch {
				throw;
			} finally {
				if (fs != null) { fs.Close(); }
			}

			// Reset the modified flag.
			_modified = false;
		}


		#region Events
		// Hook into the events.
		private void StartEventProcessing() {
			this.Closing += new CancelEventHandler(FrmMain_Closing);
			mnuFileNew.Click += new EventHandler(mnuFileNew_Click);
			mnuFileOpen.Click += new EventHandler(mnuFileOpen_Click);
			mnuFileSave.Click += new EventHandler(mnuFileSave_Click);
			mnuFileSaveAs.Click += new EventHandler(mnuFileSaveAs_Click);
			mnuFileExit.Click += new EventHandler(mnuFileExit_Click);
			mnuBlipsMainAdd.Click += new EventHandler(mnuBlipsMainAdd_Click);
			mnuBlipsMainEdit.Click += new EventHandler(mnuBlipsEdit_Click);
			mnuBlipsMainRemove.Click += new EventHandler(mnuBlipsRemove_Click);
			mnuBlipsMainImport.Click += new EventHandler(mnuBlipsMainImport_Click);
			mnuHelpAbout.Click += new EventHandler(mnuHelpAbout_Click);
			mnuBlipsEdit.Click += new EventHandler(mnuBlipsEdit_Click);
			mnuBlipsRemove.Click += new EventHandler(mnuBlipsRemove_Click);
			listBlips.ColumnClick += new ColumnClickEventHandler(listBlips_ColumnClick);
			listBlips.DoubleClick += new EventHandler(mnuBlipsEdit_Click);
		}

		// The application is closing.
		private void FrmMain_Closing(object sender, CancelEventArgs e) {
			e.Cancel = !ensureSave();
		}

		// User wants to start a new file.
		private void mnuFileNew_Click(object sender, System.EventArgs e) {
			if (ensureSave()) {
				NewFile();
			}
		}

		// User wants to open a file.
		private void mnuFileOpen_Click(object sender, System.EventArgs e) {
			if (ensureSave()) {
				dialogOpen.Title = "Open Blip File";
				if (dialogOpen.ShowDialog(this) == DialogResult.OK) {
					OpenFile(dialogOpen.FileName);
				}
			}
		}
		
		// User wants to save the file.
		private void mnuFileSave_Click(object sender, System.EventArgs e) {
			SaveFile();
		}
		
		// User wants to save the file with a new name.
		private void mnuFileSaveAs_Click(object sender, System.EventArgs e) {
			SaveFileAs();
		}

		// User wants to exit the program.
		private void mnuFileExit_Click(object sender, System.EventArgs e) {
			this.Close();
		}

		// User wants to view information about the program.
		private void mnuHelpAbout_Click(object sender, System.EventArgs e) {
			FrmAbout fa = new FrmAbout();
			fa.ShowDialog(this);
		}

		// User wants to add a blip to the file.
		private void mnuBlipsMainAdd_Click(object sender, System.EventArgs e) {
			FrmAddEditBlip faeb = new FrmAddEditBlip();

			if (faeb.ShowDialog(this) == DialogResult.OK) {
				// Create the blip.
				Blip b = new Blip();
				b.Name = faeb.BlipName;
				b.District = faeb.District;
				b.X = faeb.X;
				b.Y = faeb.Y;
				b.Z = faeb.Z;

				// Add it to the list.
				_blipCollection.Blips.Add(b);
				addBlipToList(b);
				_modified = true;
			}
		}

		// User wants to import blips from another file.
		private void mnuBlipsMainImport_Click(object sender, System.EventArgs e) {
			dialogOpen.Title = "Import Blip File";
			if (dialogOpen.ShowDialog(this) == DialogResult.OK) {
				try {
					FrmImport fi = new FrmImport(dialogOpen.FileName);
					if (fi.ShowDialog(this) == DialogResult.OK) {
						foreach (Blip b in fi.BlipCollection.Blips) {
							_blipCollection.Blips.Add(b);
						}

						// Update the list.
						_modified = true;
						refreshList();
					}
				} catch (Exception ex) {
					MessageBox.Show("There was an error when importing the blips." + Environment.NewLine + Environment.NewLine + ex.Message, "File Error");
				}
			}
		}

		// User wants to edit the selected blip.
		private void mnuBlipsEdit_Click(object sender, System.EventArgs e) {
			if (listBlips.SelectedItems.Count < 1) { return; }

			ListViewItem lvi = listBlips.SelectedItems[0];
			Blip b = (Blip)(lvi.Tag);

			FrmAddEditBlip faeb = new FrmAddEditBlip(b);
			if (faeb.ShowDialog(this) == DialogResult.OK) {
				// Update the blip.
				b.Name = faeb.BlipName;
				b.District = faeb.District;
				b.X = faeb.X;
				b.Y = faeb.Y;
				b.Z = faeb.Z;

				// Update the list.
				lvi.SubItems[0].Text = faeb.BlipName;
				lvi.SubItems[1].Text = faeb.District.ToString();
				lvi.SubItems[2].Text = faeb.X.ToString();
				lvi.SubItems[3].Text = faeb.Y.ToString();
				lvi.SubItems[4].Text = faeb.Z.ToString();

				_modified = true;
			}
		}
		
		// User wants to remove the selected blip(s) from the file.
		private void mnuBlipsRemove_Click(object sender, System.EventArgs e) {
			foreach (ListViewItem lvi in listBlips.SelectedItems) {
				_blipCollection.Blips.Remove(lvi.Tag);
				lvi.Remove();

				_modified = true;
			}
		}

		// User wants to sort the list.
		private void listBlips_ColumnClick(object sender, ColumnClickEventArgs args) {
			if (args.Column == lvColumnSorter.SortColumn) {
				if (lvColumnSorter.Order == SortOrder.Ascending) {
					lvColumnSorter.Order = SortOrder.Descending;
				} else {
					lvColumnSorter.Order = SortOrder.Ascending;
				}
			} else {
				lvColumnSorter.SortColumn = args.Column;
				lvColumnSorter.Order = SortOrder.Ascending;
			}

			listBlips.Sort();
		}
		#endregion


		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(FrmMain));
			this.mnuMain = new System.Windows.Forms.MainMenu();
			this.menuItem1 = new System.Windows.Forms.MenuItem();
			this.mnuFileNew = new System.Windows.Forms.MenuItem();
			this.mnuFileOpen = new System.Windows.Forms.MenuItem();
			this.mnuFileSave = new System.Windows.Forms.MenuItem();
			this.mnuFileSaveAs = new System.Windows.Forms.MenuItem();
			this.menuItem6 = new System.Windows.Forms.MenuItem();
			this.mnuFileExit = new System.Windows.Forms.MenuItem();
			this.menuItem2 = new System.Windows.Forms.MenuItem();
			this.mnuBlipsMainAdd = new System.Windows.Forms.MenuItem();
			this.mnuBlipsMainEdit = new System.Windows.Forms.MenuItem();
			this.mnuBlipsMainRemove = new System.Windows.Forms.MenuItem();
			this.menuItem3 = new System.Windows.Forms.MenuItem();
			this.mnuBlipsMainImport = new System.Windows.Forms.MenuItem();
			this.mnuHelp = new System.Windows.Forms.MenuItem();
			this.mnuHelpAbout = new System.Windows.Forms.MenuItem();
			this.listBlips = new System.Windows.Forms.ListView();
			this.colBlipsName = new System.Windows.Forms.ColumnHeader();
			this.colBlipsDistrict = new System.Windows.Forms.ColumnHeader();
			this.colBlipsX = new System.Windows.Forms.ColumnHeader();
			this.colBlipsY = new System.Windows.Forms.ColumnHeader();
			this.colBlipsZ = new System.Windows.Forms.ColumnHeader();
			this.mnuBlips = new System.Windows.Forms.ContextMenu();
			this.mnuBlipsEdit = new System.Windows.Forms.MenuItem();
			this.mnuBlipsRemove = new System.Windows.Forms.MenuItem();
			this.dialogOpen = new System.Windows.Forms.OpenFileDialog();
			this.dialogSave = new System.Windows.Forms.SaveFileDialog();
			this.SuspendLayout();
			// 
			// mnuMain
			// 
			this.mnuMain.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					this.menuItem1,
																					this.menuItem2,
																					this.mnuHelp});
			// 
			// menuItem1
			// 
			this.menuItem1.Index = 0;
			this.menuItem1.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					  this.mnuFileNew,
																					  this.mnuFileOpen,
																					  this.mnuFileSave,
																					  this.mnuFileSaveAs,
																					  this.menuItem6,
																					  this.mnuFileExit});
			this.menuItem1.Text = "&File";
			// 
			// mnuFileNew
			// 
			this.mnuFileNew.Index = 0;
			this.mnuFileNew.Shortcut = System.Windows.Forms.Shortcut.CtrlN;
			this.mnuFileNew.Text = "&New";
			// 
			// mnuFileOpen
			// 
			this.mnuFileOpen.Index = 1;
			this.mnuFileOpen.Shortcut = System.Windows.Forms.Shortcut.CtrlO;
			this.mnuFileOpen.Text = "&Open...";
			// 
			// mnuFileSave
			// 
			this.mnuFileSave.Index = 2;
			this.mnuFileSave.Shortcut = System.Windows.Forms.Shortcut.CtrlS;
			this.mnuFileSave.Text = "&Save";
			// 
			// mnuFileSaveAs
			// 
			this.mnuFileSaveAs.Index = 3;
			this.mnuFileSaveAs.Text = "Save &As...";
			// 
			// menuItem6
			// 
			this.menuItem6.Index = 4;
			this.menuItem6.Text = "-";
			// 
			// mnuFileExit
			// 
			this.mnuFileExit.Index = 5;
			this.mnuFileExit.Text = "E&xit";
			// 
			// menuItem2
			// 
			this.menuItem2.Index = 1;
			this.menuItem2.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					  this.mnuBlipsMainAdd,
																					  this.mnuBlipsMainEdit,
																					  this.mnuBlipsMainRemove,
																					  this.menuItem3,
																					  this.mnuBlipsMainImport});
			this.menuItem2.Text = "&Blips";
			// 
			// mnuBlipsMainAdd
			// 
			this.mnuBlipsMainAdd.Index = 0;
			this.mnuBlipsMainAdd.Text = "&Add Blip...";
			// 
			// mnuBlipsMainEdit
			// 
			this.mnuBlipsMainEdit.Index = 1;
			this.mnuBlipsMainEdit.Text = "&Edit Blip...";
			// 
			// mnuBlipsMainRemove
			// 
			this.mnuBlipsMainRemove.Index = 2;
			this.mnuBlipsMainRemove.Text = "&Remove Blip";
			// 
			// menuItem3
			// 
			this.menuItem3.Index = 3;
			this.menuItem3.Text = "-";
			// 
			// mnuBlipsMainImport
			// 
			this.mnuBlipsMainImport.Index = 4;
			this.mnuBlipsMainImport.Text = "&Import Blips...";
			// 
			// mnuHelp
			// 
			this.mnuHelp.Index = 2;
			this.mnuHelp.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					this.mnuHelpAbout});
			this.mnuHelp.Text = "&Help";
			// 
			// mnuHelpAbout
			// 
			this.mnuHelpAbout.Index = 0;
			this.mnuHelpAbout.Text = "&About";
			// 
			// listBlips
			// 
			this.listBlips.Columns.AddRange(new System.Windows.Forms.ColumnHeader[] {
																						this.colBlipsName,
																						this.colBlipsDistrict,
																						this.colBlipsX,
																						this.colBlipsY,
																						this.colBlipsZ});
			this.listBlips.ContextMenu = this.mnuBlips;
			this.listBlips.Dock = System.Windows.Forms.DockStyle.Fill;
			this.listBlips.FullRowSelect = true;
			this.listBlips.Location = new System.Drawing.Point(0, 0);
			this.listBlips.Name = "listBlips";
			this.listBlips.Size = new System.Drawing.Size(580, 281);
			this.listBlips.TabIndex = 0;
			this.listBlips.View = System.Windows.Forms.View.Details;
			// 
			// colBlipsName
			// 
			this.colBlipsName.Text = "Name";
			this.colBlipsName.Width = 200;
			// 
			// colBlipsDistrict
			// 
			this.colBlipsDistrict.Text = "District";
			this.colBlipsDistrict.Width = 120;
			// 
			// colBlipsX
			// 
			this.colBlipsX.Text = "X";
			this.colBlipsX.Width = 80;
			// 
			// colBlipsY
			// 
			this.colBlipsY.Text = "Y";
			this.colBlipsY.Width = 80;
			// 
			// colBlipsZ
			// 
			this.colBlipsZ.Text = "Z";
			this.colBlipsZ.Width = 80;
			// 
			// mnuBlips
			// 
			this.mnuBlips.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					 this.mnuBlipsEdit,
																					 this.mnuBlipsRemove});
			// 
			// mnuBlipsEdit
			// 
			this.mnuBlipsEdit.DefaultItem = true;
			this.mnuBlipsEdit.Index = 0;
			this.mnuBlipsEdit.Text = "Edit Blip";
			// 
			// mnuBlipsRemove
			// 
			this.mnuBlipsRemove.Index = 1;
			this.mnuBlipsRemove.Text = "Remove Blip";
			// 
			// dialogOpen
			// 
			this.dialogOpen.DefaultExt = "cfg";
			this.dialogOpen.FileName = "pblip.cfg";
			this.dialogOpen.Filter = "MXO Config Files (*.cfg)|*.cfg|All Files (*.*)|*.*";
			this.dialogOpen.ShowReadOnly = true;
			this.dialogOpen.Title = "Open Blip File";
			// 
			// dialogSave
			// 
			this.dialogSave.DefaultExt = "cfg";
			this.dialogSave.FileName = "pblip.cfg";
			this.dialogSave.Filter = "MXO Config Files (*.cfg)|pblip.cfg|All Files (*.*)|*.*";
			this.dialogSave.Title = "Save Blip File";
			// 
			// FrmMain
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(580, 281);
			this.Controls.Add(this.listBlips);
			this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
			this.Menu = this.mnuMain;
			this.Name = "FrmMain";
			this.Text = "Blipper";
			this.ResumeLayout(false);

		}
		#endregion
	}
}
